@@ -788,6 +788,7 @@ commit_cb (GObject      *object,
 
 	if (error) {
 		g_critical ("Could not commit: %s", error->message);
+		g_error_free (error);
 	}
 }
 
@@ -855,7 +856,6 @@ sparql_update_cb (GObject      *object,
 	TrackerMinerFS *fs;
 	TrackerMinerFSPrivate *priv;
 	ProcessData *data;
-
 	GError *error = NULL;
 
 	tracker_miner_execute_update_finish (TRACKER_MINER (object), result, &error);
@@ -866,6 +866,7 @@ sparql_update_cb (GObject      *object,
 
 	if (error) {
 		g_critical ("Could not execute sparql: %s", error->message);
+		g_error_free (error);
 	} else {
 		if (fs->private->been_crawled) {
 			/* Only commit immediately for
@@ -887,14 +888,21 @@ sparql_query_cb (GObject      *object,
                  gpointer      user_data)
 {
 	SparqlQueryData *data = user_data;
-	TrackerMiner *miner = TRACKER_MINER (object);
-
+	const GPtrArray *query_results;
+	TrackerMiner *miner;
 	GError *error = NULL;
 
-	const GPtrArray *query_results = tracker_miner_execute_sparql_finish (miner, result, &error);
+	miner = TRACKER_MINER (object);
+	query_results = tracker_miner_execute_sparql_finish (miner, result, &error);
+
+	if (error) {
+		g_critical ("Could not execute sparql query: %s", error->message);
+		g_error_free (error);
+	}
 
 	data->value = query_results && query_results->len == 1;
 	g_main_loop_quit (data->main_loop);
+
 }
 
 static void
@@ -1044,6 +1052,7 @@ item_remove (TrackerMinerFS *fs,
 
 	if (!item_query_exists (fs, file)) {
 		g_debug ("  File does not exist anyway (uri:'%s')", uri);
+		g_free (uri);
 		return TRUE;
 	}
 
@@ -1189,6 +1198,7 @@ item_update_uri_recursively_cb (GObject      *object,
 
 	if (error) {
 		g_critical ("Could not query children: %s", error->message);
+		g_error_free (error);
 	} else {
 		if (query_results) {
 			gint i;
@@ -1302,8 +1312,8 @@ item_move (TrackerMinerFS *fs,
 	g_string_append_printf (sparql, " INSERT INTO <%s> {", uri);
 
 	escaped_filename = g_strescape (g_file_info_get_display_name (file_info), NULL);
-
 	g_string_append_printf (sparql, " <%s> nfo:fileName \"%s\" .", source_uri, escaped_filename);
+	g_free (escaped_filename);
 
 	move_data.main_loop = g_main_loop_new (NULL, FALSE);
 	move_data.level = 0;
@@ -1446,7 +1456,8 @@ item_queue_handlers_cb (gpointer user_data)
 	fs = user_data;
 	queue = item_queue_get_next_file (fs, &file, &source_file);
 
-	if (file && tracker_file_is_locked (file)) {
+	if (file && queue != QUEUE_DELETED &&
+	    tracker_file_is_locked (file)) {
 		/* File is locked, ignore any updates on it */
 		g_object_unref (file);
 
