@@ -308,8 +308,14 @@ folder_registry_new (const gchar *account_uri,
 }
 
 static void
-on_replied (GError *error, gpointer user_data) 
-{ 
+on_replied (GError   *error,
+	    gpointer  user_data)
+{
+	if (error) {
+		g_warning ("Error updating data: %s\n", error->message);
+		g_error_free (error);
+	}
+
 	g_object_unref (user_data);
 }
 
@@ -319,9 +325,10 @@ send_sparql_update (TrackerEvolutionPlugin *self, const gchar *sparql)
 	TrackerEvolutionPluginPrivate *priv = TRACKER_EVOLUTION_PLUGIN_GET_PRIVATE (self);
 
 	if (priv->client) {
-		tracker_resources_batch_sparql_update_async (priv->client, sparql,
-		                                             on_replied, 
-		                                             g_object_ref (self));
+		dbus_g_proxy_call_no_reply (priv->client->proxy_resources,
+					    "SparqlUpdate",
+					    G_TYPE_STRING, sparql,
+					    G_TYPE_INVALID);
 	}
 }
 
@@ -333,16 +340,15 @@ send_sparql_commit (TrackerEvolutionPlugin *self, gboolean update)
 	if (priv->client) {
 		if (update) {
 			gchar *date_s = tracker_date_to_string (time (NULL));
-			gchar *update = g_strdup_printf ("DELETE { <" DATASOURCE_URN "> nie:contentLastModified ?d } "
+			gchar *update = g_strdup_printf ("DELETE FROM <"DATASOURCE_URN"> { <" DATASOURCE_URN "> nie:contentLastModified ?d } "
 			                                 "WHERE { <" DATASOURCE_URN "> a nie:InformationElement ; nie:contentLastModified ?d } \n"
-			                                 "INSERT { <" DATASOURCE_URN "> nie:contentLastModified \"%s\" }",
+			                                 "INSERT INTO <"DATASOURCE_URN"> { <" DATASOURCE_URN "> nie:contentLastModified \"%s\" }",
 			                                 date_s);
-			g_free (date_s);
-			tracker_resources_batch_sparql_update_async (priv->client, update, 
-			                                             on_replied, 
-			                                             g_object_ref (self));
+
+			send_sparql_update (self, update);
 
 			g_free (update);
+			g_free (date_s);
 		}
 
 		tracker_resources_batch_commit_async (priv->client, on_replied, 
@@ -646,7 +652,7 @@ on_folder_summary_changed (CamelFolder *folder,
 
 			tracker_sparql_builder_drop_graph (sparql, uri);
 
-			tracker_sparql_builder_insert_open (sparql);
+			tracker_sparql_builder_insert_open (sparql, uri);
 
 			process_fields (sparql, uid, flags, sent, subject, 
 			                from, to, cc, size, folder, uri);
@@ -692,16 +698,9 @@ on_folder_summary_changed (CamelFolder *folder,
 
 			send_sparql_update (info->self, tracker_sparql_builder_get_result (sparql));
 
-/*
-			I saw a crash here, can't figure out why, it was
-			in strcmp of g_object_set (maybe multi-threading
-			of Camel is involved?)
-
-			g_object_set (info->self, "progress", 
-			              (gdouble) (merged->len - i) / merged->len,
-			              "status", _("Updating"), 
+			g_object_set (info->self, "progress",
+			              (gdouble) i / merged->len,
 			              NULL);
-*/
 
 			g_object_unref (sparql);
 
@@ -718,29 +717,34 @@ on_folder_summary_changed (CamelFolder *folder,
 	/* the uid_removed member contains the removed-from-the-summary items */
 
 	if (changes->uid_removed && changes->uid_removed->len > 0) {
-		GString *sparql = g_string_new ("DELETE { \n");
+
+		/* The FROM uri is not exactly right here, but we just want 
+		 * graph != NULL in tracker-store/tracker-writeback.c */
+
+		GString *sparql = g_string_new ("");
 
 		for (i = 0; i< changes->uid_removed->len; i++) {
 
 			/* This is not a path but a URI, don't use the OS's 
 			 * directory separator here */
 
-			g_string_append_printf (sparql, "\t<%s%s/%s> a rdfs:Resource . \n", 
+			g_string_append_printf (sparql, "DELETE FROM <%s%s/%s> { <%s%s/%s> a rdfs:Resource }\n ", 
+			                        em_uri, 
+			                        camel_folder_get_full_name (folder),
+			                        (char*) changes->uid_removed->pdata[i],
 			                        em_uri, 
 			                        camel_folder_get_full_name (folder),
 			                        (char*) changes->uid_removed->pdata[i]);
 		}
 
-		g_string_append_c (sparql, '}');
-
 		send_sparql_update (info->self, sparql->str);
 		g_string_free (sparql, TRUE);
 	}
 
 	send_sparql_commit (info->self, TRUE);
 
-	/* g_object_set (info->self, "progress", 
-	              1.0, NULL); */
+	g_object_set (info->self, "progress",
+	              1.0, NULL);
 
 	g_free (em_uri);
 }
@@ -806,10 +810,12 @@ many_idle_handler (gpointer user_data)
 				priv->total_popped = priv->of_total;
 			}
 
-			g_object_set (user_data, "progress", 
-			              (gdouble) (priv->of_total - priv->total_popped) / priv->of_total,
-			              "status", _("Updating"), 
+#if 0
+			g_object_set (user_data, "progress",
+			              (gdouble) priv->total_popped / priv->of_total,
+			              "status", _("Updating"),
 			              NULL);
+#endif
 
 			send_sparql_update (user_data, query);
 		} else {
@@ -953,7 +959,7 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 
 					tracker_sparql_builder_drop_graph (sparql, uri);
 
-					tracker_sparql_builder_insert_open (sparql);
+					tracker_sparql_builder_insert_open (sparql, uri);
 
 					process_fields (sparql, uid, flags, sent, 
 					                subject, from, to, cc, size, 
@@ -1023,6 +1029,8 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 						g_free(name);
 						g_free(value);
 					}
+
+					g_free (uri);
 					g_free (p);
 
 					tracker_sparql_builder_insert_close (sparql);
@@ -1075,11 +1083,13 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 
 				priv->of_total++;
 
+#if 0
 				if (priv->of_total > priv->total_popped) {
-					g_object_set (self, "progress", 
-					              (gdouble) (priv->of_total - priv->total_popped) / priv->of_total,
+					g_object_set (self, "progress",
+					              (gdouble) priv->total_popped / priv->of_total,
 					              NULL);
 				}
+#endif
 
 				if (start_handler) {
 					start_many_handler (self);
@@ -1186,10 +1196,14 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 		}
 
 		if (count > 0) {
-			GString *sparql = g_string_new ("DELETE { \n");
+			/* The FROM uri is not exactly right here, but we just want 
+			 * graph != NULL in tracker-store/tracker-writeback.c */
+
+			GString *sparql = g_string_new ("");
 
 			for (i = 0; i < subjects_a->len; i++) {
-				g_string_append_printf (sparql, "\t<%s> a rdfs:Resource . \n", 
+				g_string_append_printf (sparql, "DELETE FROM <%s> { <%s> a rdfs:Resource } \n", 
+				                        (gchar *) g_ptr_array_index (subjects_a, i),
 				                        (gchar *) g_ptr_array_index (subjects_a, i));
 			}
 
@@ -1651,7 +1665,7 @@ register_client_second_half (ClientRegistry *info)
 
 	if (info->last_checkout < too_old) {
 
-		send_sparql_update (info->self, "DELETE { ?s a rdfs:Resource } "
+		send_sparql_update (info->self, "DELETE FROM <"DATASOURCE_URN"> { ?s a rdfs:Resource } "
 		                                "WHERE { ?s nie:dataSource <" DATASOURCE_URN "> }");
 		send_sparql_commit (info->self, FALSE);
 
@@ -2095,14 +2109,14 @@ tracker_evolution_plugin_finalize (GObject *plugin)
 	g_static_rec_mutex_lock (priv->mutex);
 
 	if (priv->registered_folders) {
-		g_hash_table_destroy (priv->registered_folders);
-		g_hash_table_destroy (priv->cached_folders);
+		g_hash_table_unref (priv->registered_folders);
+		g_hash_table_unref (priv->cached_folders);
 		priv->cached_folders = NULL;
 		priv->registered_folders = NULL;
 	}
 
 	if (priv->registered_stores) {
-		g_hash_table_destroy (priv->registered_stores);
+		g_hash_table_unref (priv->registered_stores);
 		priv->registered_stores = NULL;
 	}
 
