@@ -449,6 +449,7 @@ set_up_mount_point_cb (GObject      *source,
 	if (error) {
 		g_critical ("Could not set up mount point '%s': %s",
 			    removable_device_urn, error->message);
+		g_error_free (error);
 	}
 
 	g_free (removable_device_urn);
@@ -541,6 +542,7 @@ init_mount_points_cb (GObject      *source,
                       gpointer      user_data)
 {
 	GError *error = NULL;
+
 	tracker_miner_execute_update_finish (TRACKER_MINER (source),
 	                                     result,
 	                                     &error);
@@ -548,6 +550,7 @@ init_mount_points_cb (GObject      *source,
 	if (error) {
 		g_critical ("Could not initialize currently active mount points: %s",
 			    error->message);
+		g_error_free (error);
 	}
 }
 
@@ -564,7 +567,7 @@ query_mount_points_cb (GObject      *source,
 	GString *accumulator;
 	gint i;
 	GError *error = NULL;
-	GPtrArray *query_results;
+	const GPtrArray *query_results;
 #ifdef HAVE_HAL
 	GSList *udis, *u;
 #endif
@@ -574,6 +577,7 @@ query_mount_points_cb (GObject      *source,
 	                                                     &error);
 	if (error) {
 		g_critical ("Could not obtain the mounted volumes: %s", error->message);
+		g_error_free (error);
 		return;
 	}
 
@@ -1301,6 +1305,7 @@ process_file_cb (GObject      *object,
 		/* Something bad happened, notify about the error */
 		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), file, error);
 		process_file_data_free (data);
+		g_error_free (error);
 		return;
 	}
 
